;*********************************************************************************************************
;*******  Rechenoperationen zur Lampensteuerung Version 2.2					**********
;*********************************************************************************************************

;*********************************************************************************************************
;*******  8-Bit vorzeichenlose Multiplikation in Software					**********
;*******  bentigte Register: Eingangswerte = var10 * var11					**********
;*******				 Ergebnis = r[1:0]					**********
;*******	Hilfsregister = lc, Eingangsregister var11 wird zerstrt			**********
;*********************************************************************************************************
mul8u:   
	clr     r1			;Ergebnisregister lschen
        ldi     lc,8			;Schleifenzhler laden
m8u_1:
	lsr     var11			;niederwertigstes Bit Multiplikant ins Carry schieben
        brcc    m8u_2			;wenn 0, keine Addition
        add     r1,var10		;sonst Multiplikator ins Highbyte addieren
m8u_2:
	ror     r1			;Produkt nach rechts schieben
        ror     r0			;
        dec     lc			;Das ganze 8 mal
        brne    m8u_1			;
	ret

;*********************************************************************************************************
;*******  16-Bit vorzeichenlose Multiplikation in Software					**********
;*******  bentigte Register: Eingangswerte = var1[1:0] * var2[1:0]				**********
;*******			   Ergebnis = r[3:0]						**********
;*******	Hilfsregister = lc, Eingangsregister var2[1:0] wird zerstrt			**********
;*********************************************************************************************************
mul16u:   
	clr     r3			;Ergebnisregister lschen
	clr     r2			;
	clr     r1			;
        ldi     lc,16			;Schleifenzhler laden
m16u_1:
	lsr     var21			;niederwertigstes Bit Multiplikant ins Carry schieben
	ror	var20			;
        brcc    m16u_2			;wenn 0, keine Addition
        add     r2,var10		;sonst Multiplikator in Highbytes addieren
        adc     r3,var11		;
m16u_2:
	ror     r3			;Produkt nach rechts schieben
	ror     r2			;
	ror     r1			;
	ror     r0			;
        dec     lc			;Das ganze 16 mal
        brne    m16u_1			;
	ret

;*********************************************************************************************************
;*******  24-Bit vorzeichenlose Division							**********
;*******  bentigte Register: Eingangswerte = var1[2:0] / var2[2:0]				**********
;*******			   Ergebnis = var1[2:0]						**********
;*******	Hilfsregister = lc, r[2:0], Eingangsregister var2[1:0] wird nicht zerstrt	**********
;*******	Laufzeit = 348..412 cycles (+ret), Quelle: http://elm-chan.org/cc_e.html	**********
;*********************************************************************************************************
div24u:
	clr	r0		;initialize variables
	clr	r1		;  mod = 0;
	clr	r2		;  lc = 24;
	ldi	lc,24		;/
d24u_1:				;---- calcurating loop
	lsl	var10		;var1 = var1 << 1;
	rol	var11		;
	rol	var12		;/
	rol	r0		;mod = mod << 1 + carry;
	rol	r1		;
	rol	r2		;/
	cp	r0,var20	;if (mod => var2) {
	cpc	r1,var21	; mod -= var2; var1++;
	cpc	r2,var22	; }
	brcs	d24u_2		;
	inc	var10		;
	sub	r0,var20	;
	sbc	r1,var21	;
	sbc	r2,var22	;
d24u_2:
	dec	lc		;if (--lc > 0)
	brne	d24u_1		; continue loop;
	ret

