;*********************************************************************************************************
;*******	Projekt:  Steuerung fr batteriebetriebene Unterwasserleuchte			**********
;*********************************************************************************************************
;*******	Hardware: Attiny25, Leistungs-FET, Signalgeber					**********
;*********************************************************************************************************
;*******	Idee, Hardware, Software:	Stefan Kneller (C) 2006 - 2022			**********
;*******	nichtgewerbliche Nutzung fr private Einzelstcke gestattet			**********
;*********************************************************************************************************
;*******	Todo:
;*******	- 
;*******	- 
;*********************************************************************************************************
;*******	Wunschliste:
;*******	- anlernbare Akku-Nennspannung verworfen weil das mehr Risiken als Nutzen bringt 
;*******	  
;*********************************************************************************************************
.include "tn25def.inc"		;wegen Codegre fr Attiny 15 nicht mehr nutzbar
;*********************************************************************************************************
;*******	ASCII Klartext fr das Ende des HEX-Files					**********
;*********************************************************************************************************
.equ	Kommentarzeilen	= 5
.org	flashend-(8*kommentarzeilen-1)	;so liegt der Kommentar am Flashende

.db	" Steuerung fuer "
.db	" Handlampe V2.2 "
.db	"Stand 27.03.2022"
.db	" Fuse-High: DC  "
.db	" Fuse-low:  E2  "

;*********************************************************************************************************
;*******  ber den Hauptschalter wird die Lampe eingeschaltet und leuchtet mit 100% Helligkeit.
;*******  Es findet ein Softstart von einer Sekunde statt um den Glhwendel zu schonen.
;*******  
;*******  Der Benutzer kann mittels des Lampenschalters Kommandos zum Helligkeitswechsel geben.
;*******  
;*******  Bei gewhlter Betriebsart Schalten:
;*******  Durch kurzes Aus- und Wiedereinschalten am Hauptschalter senkt die Lampe die Helligkeit auf die
;*******  nchst kleinere Stufe. Ist die kleinste Stufe erreicht, wird auf maximale Helligkeit geschaltet.
;*******  Ist die Akkuspannung grer als die Nennspannung des Brenners betrgt die maximale Helligkeit 125%.
;*******  Whrend des Schalterkommandos verlscht die Lampe nicht. 
;*******  
;*******  Bei gewhlter Betriebsart Dimmen:
;*******  Durch kurzes Aus- und Wiedereinschalten am Hauptschalter senkt die Lampe langsam die Helligkeit
;*******  um ca. 10% pro Sekunde.
;*******  Durch nochmaliges kurzes Aus- und Wiedereinschalten am Hauptschalter wird die aktuelle Helligkeit gehalten.
;*******  Wird kein Stoppkommando ausgefhrt luft die Helligkeit bis zum Minimum und stoppt dort.
;*******  Beim nchsten Schalterkommando wird die Helligkeit langsam erhht.
;*******  Wird kein Stoppkommando ausgefhrt luft die Helligkeit bis zum Maximum und stoppt dort.
;*******  Der Vorgang kann beliebig oft wiederholt werden.
;*******  Die letzte eingestellte Helligkeit wird beim Ausschalten der Lampe nicht gespeichert.
;*******  
;*******  Durch ein langes Aus- und Wiedereinschalten des Hauptschalters von mehr als 0,6 Sekunden geht die Lampe
;*******  in den SOS-Modus und sendet die Zeichenfolge S O S (... --- ...) als Morsecode mit maximaler Helligkeit.
;*******  In diesem Modus finden keine Unterspannungs- oder Wassereinbruchsprfungen und -warnungen statt.
;*******  Dieser Modus bleibt aktiv bis die Lampe abgeschaltet wird oder der Akku vollstndig entladen ist.
;*******  
;*******  Bei sich nherndem Ende der Akkulaufzeit wird duch Abblenden der Helligkeit in Intervallen von 4 Sekunden
;*******  vor dem nahen Ende der Betriebszeit der Lampe gewarnt.
;*******  Die Vorwarnspannung liegt bei 107% der Abschaltspannung.
;*******  Durch Wechsel in eine andere Helligkeit wird der Alarm gelscht.
;*******  Falls sich dadurch die Akkuspannung nicht ausreichend erholt wird nach einer 16 Sekunden langen Sperre
;*******  erneut Unterspannung signalisiert.
;*******  Bei entladenem Akku schaltet die Lampe ab und erinnert durch Piepstne daran den Hauptschalter abzuschalten.
;*******  Sie lsst sich durch Wiedereinschalten erneut fr 16 Sekunden nutzen bis die Akkuspannung auch fr dieses "Notlicht" zu gering wird.
;*******  
;*******  Wird der Wassereinbruchsensor durch eingelaufenes Wasser auf ein Potential ber 0,5 V gezogen,
;*******  wird alle 4 Sekunden durch kurzes Absenken der Helligkeit und 2 lange Piepstne alarmiert.
;*******  Der Alarm kann durch ein kurzes Schalterkommando gelscht werden, tatschlich eingedrungenes Wasser wird ihn
;*******  jedoch wieder auslsen
;*******  Unterspannungswarnungen finden nicht statt, die Abschaltung bei Unterspannung ist jedoch aktiv.
;*******  
;*******  Es knnen zur Laufzeit des Programms verschiedene Optionen zu- und abgewhlt werden.
;*******  
;*******  Men 1: Helligkeitseinstellung in Stufen oder stufenlos
;*******  Men 2: Brennerleistung bei wechselnder Akkuspannung konstant halten oder nicht
;*******  Men 3: Wassereinbruchprfung aktiviert oder deaktiviert (bei Nutzung als berwasserlampe)
;*******  Men 4: neue Abschaltspannung lernen
;*******  Men 5: Abschaltspannung auf Werkseinstellung zurcksetzen
;*******  
;*******  Um in den Lernmodus zu gelangen Pin 5 (Port B0) des Controllers von der Masse trennen
;*******  (eine IC-Fassung bei der der entsprechende Kontakt entfernt ist zwischenstecken)
;*******  und die Versorgungsspannung anlegen.
;*******
;*******  Wir sind nun im Menpunkt 1, Helligkeitseinstellung.
;*******  Der Controller signalisiert Menpunkt und gewhlte Option mit einer Tonfolge:
;*******  1 kurzer Pieps fr Menpunkt 1 und 1 kurzen Pieps fr stufige oder 2 kurze Piepser fr stufenlose Einstellung
;*******  Nun kann mit einem Kurzkommando die Option gewechselt werden.
;*******  Die Quittierung erfolgt mit der neuen Tonfolge fr den Menpunkt 1 und die aktive Option.
;*******  
;*******  Mit einem langen Schalterkommando gelangt man in Menpunkt 2, Brenner auf Konstante Leistung regeln.
;*******  Der Controller signalisiert Menpunkt und gewhlte Option mit einer Tonfolge:
;*******  2 kurze Piepser fr Menpunkt 2 und 1 kurzen Pieps fr keine Regelung oder 2 kurze Piepser fr Regelung.
;*******  Nun kann mit einem Kurzkommando die Option gewechselt werden.
;*******  Die Quittierung erfolgt mit der Tonfolge fr den Menpunkt 2 und die aktive Option.
;******* 
;*******  Mit einem langen Schalterkommando gelangt man in Menpunkt 3, Wassereinbruchprfung.
;*******  Der Controller signalisiert Menpunkt und gewhlte Option mit einer Tonfolge:
;*******  3 kurze Piepser fr Menpunkt 3 und 1 kurzer Pieps fr aktiven Wassereinbruchtest oder
;*******  2 kurze Piepser fr deaktivierten Wassereinbruchtest.
;*******  Nun kann mit einem Kurzkommando die Option gewechselt werden.
;*******  Die Quittierung erfolgt mit der Tonfolge fr den Menpunkt 3 und die aktive Option.
;*******  
;*******  Mit einem langen Schalterkommando gelangt man in Menpunkt 4, neue Abschaltspannung lernen.
;*******  Der Controller signalisiert den Menpunkt 4 mit 4 kurzen Piepsern.
;*******  Fr das Anlernen einer neuen Abschaltspannung die Betriebsspannung auf den gewnschten Wert einstellen
;*******  und mit einem Kurzkommando die bernahme als neue Abschaltspannung anfordern.
;*******  Es wird nun solange gewartet, bis die Betriebsspannung stabil ist. Dabei ertnt jede Sekunde ein Pieps.
;*******  Sobald eine stabile Spannung erkannt und als neue Abschaltspannung abgespeichert wurde wird mit der Tonfolge fr
;*******  den Menpunkt 4 und 1 langen Piepser quittiert.
;*******  
;*******  Mit einem langen Schalterkommando gelangt man in Menpunkt 5, Werkseinstellung Abschaltspannung laden.
;*******  Der Controller signalisiert den Menpunkt 5 mit 5 Piepsern.
;*******  Fr das Rckstellen auf die Werkseinstellung ein kurzes Schalterkommando geben.
;*******  Die Quittierung erfolgt mit der Tonfolge fr den Menpunkt 5 und 1 langen Pieps.
;*******  
;*******  Das nchste Langkommando schaltet wieder in Menpunkt 1. 
;*******  Die Einstellungen knnen so oft gendert werden, bis sie den persnlichen Wnschen entsprechen.
;*******  
;*******  Zum Verlassen des Lernmodus die Spannungsquelle entfernen und Pin 5 wieder mit Masse verbinden.
;*******  (Adapterfassung entfernen)
;*******  
;*******  Die Spannungsmessungen am Akku erfolgen 100 mal pro Sekunde und am Wassereinbruchsensor einmal pro Sekunde
;*******  Der Lampenstrom ist whrend der Messung der Akkuspannung abgeschaltet, um Messfehler
;*******  durch bergangswiderstnde zu verringern.
;*******  Beide Messwerte durchlaufen vor ihrer Auswertung Filter.
;*******  
;*******  Der Dimm- oder Schaltvorgang (je nach Option) beeinflusst die "Nutzerhelligkeit"
;*******  in den Grenzen min-Helligkeit bis max-Helligkeit.
;*******  
;*******  Die "Nutzerhelligkeit" wird 25 mal pro Sekunde an die "Sollhelligkeit" bergeben.
;*******  Bei Option "Leistungsregelung" wird vor der bergabe an eine wechselnde Akkuspannung nachgeregelt.
;*******  80% der Sollhelligkeit werden als Alarmhelligkeit bergeben.
;*******
;*******  Der Controllertakt ist von 8 auf 6,5536 MHz herruntergetrimmt.
;*******  Brown-Out Detektor bei 4 Volt Schwelle schtzt den FET whrend Reset.
;*******	
;*******	- Kalibrierung des AD-Wandlers:
;*******	  die Konstante "Widerstand_1" so ndern, dass ein 12 Volt Brenner 
;*******	  bei 12 Volt Betriebsspannung mit 100% PWM angesteuert wird
;*******	  eine Verkleinerung der Konstante "Widerstand_1" vergert die PWM
;*******	
;*******  Die Lampensteuerung ist verpolsicher aber nicht berlast- und kurzschlussfest.
;*******
;*********************************************************************************************************
;*******	Pinbelegung									**********
;*********************************************************************************************************
;                              1  _____
; Reset                       - /     |-   +5 Volt
; Eingang Wassereinbruch B3   -|ATTINY|-   B2 Ausgang Piepser
; Eingang Akkuspannung   B4   -|  25  |-   B1 Ausgang PWM
; Masse	                      -|______|-   B0 Eingang Lernmodus

;*********************************************************************************************************
;*******	Portzuweisungen									**********
;*********************************************************************************************************

.equ	E_Lernen	= PB0		;Eingang Programmiermodus, high-aktiv
.equ	A_PWM		= PB1		;Ausgang PWM, high aktiv
.equ	A_Piepser	= PB2		;Ausgang Piepser, low-aktiv
;	E_Wasser	= PB3		;Eingang Wassersensor
;	E_Akku		= PB4		;Eingang Akkuspannung ber Spannungsteiler R1/R2
;*********************************************************************************************************
;*******	Definition der Konstanten							**********
;*******	WB = Wertebereich								**********
;*********************************************************************************************************
.equ	U_Akku		= 12000		;Akku-Nennspannung 			} WB: 9000 - 18000 mV
.equ	U_Absch		= 10500		;Abschaltspannung, Defaultwert		} WB: 9000 - 18000 mV
.equ	U_Brenner	= 12000		;Brenner-Nennspannung			} WB: 6000 - 18000 mV
.equ	Widerstand_1	= 90		;Widerstand R1 in KOhm	} Spannungsteiler am AD-Wandler-Eingang
.equ	Widerstand_2	= 15		;Widerstand R2 in KOhm	} mit R1 kalibrieren, siehe Anleitung
.equ	k_0		= 100*u_akku*u_akku/(u_brenner*u_brenner)	;} 
.equ	k_1		= 1024*widerstand_2/(widerstand_1+widerstand_2)	;} Konstanten
.if	(k_0 > 111)			;bei Akku-Nennspannung > 1,11 * Brennerspannung
.equ	max_Helligkeit		= 125	;erlaube Lampenleistung von 125%
.else					;sonst
.equ	max_Helligkeit		= 100	;ist bei 100% Schluss
.endif
.equ	min_Helligkeit		= 50	;50% Minimumhelligkeit
.equ	alm_Helligkeit		= 80	;80% Alarmierungshelligkeit
.equ	Abstufung		= 80	;nchst niedrigere Helligkeitsstufe in % der Aktuellen  WB: 1 - 99 %
.equ	Fading_Start		= 1000	;Softstart					  WB: 10 - 2560 ms
.equ	Fading_Schalten		= 500	;bergangsdauer zwischen den Helligkeitsstufen	  WB: 10 - 2560 ms
.equ	Fading_Dimmen		= 10	;nderungsrate des Dimmvorganges		  WB: 5 - 1275% pro Sekunde, Auflsung 5%
.equ	Prellen			= 2	;Zeitdauer die als Tasterprellen gelten 	  WB: 1 - 7 Frames
.equ	Schwelle_Schalter_ein	= 500	;Pegel, ber dem der Schalter als EIN erkannt wird } WB: 0 - 2560 mV 
.equ	Dauer_Kurzkommando	= 800	;max. Schalterpause fr Erkennung "Kurzkommando" } WB: 0 - 10200 Millisekunden
.equ	Dauer_Langkommando	= 1600	;max. Schalterpause fr Erkennung "Langkommando" } WB: 0 - 10200 Millisekunden
.equ	Schwelle_Wassereinbruch	= 500	;Pegel, ber dem Wassereinbruch erkannt wird	   } Auflsung 3 mV
.equ	Filtertiefe_Akku	= 256	;Tiefpass Betriebsspannungsmessung	} WB: 256,128,64,32,16,8
.equ	Filtertiefe_Wasser	= 8	;Tiefpass Wassereinbruchsensor		} WB: 2,4,8,16 Sekunden Zeitkonstante
.equ	default_Option		= (1<<leistung_konst)	;Fabrikeinstellung Optionen
					;Lager: (1<<leistung_konst)|(1<<modus_dimmen)|(1<<kein_wasser)
.equ	Dauer_kurzer_Pieps	= 5	; }
.equ	Pause_kurzer_Pieps	= 5	; } Lnge in Frames
.equ	Dauer_langer_Pieps	= 15	; } 
.equ	Pause_langer_Pieps	= 8	; } 
.equ	adr_eeprom		= 2	;unterste genutzte Speicheradresse EEPROM
.equ	CalibrateOsz		= 0x50	;(40/53) auf 6,5536 MHz kalibrieren fr einigermaen genaue Sekunde
;.equ	Clk			= 6553600;Controllertakt in Hz

;*********************************************************************************************************
;*******	Registerbelegung      								**********
;*******	Befehle, die mit den Low-Registern r0 bis r15 nicht mglich sind:		**********
;*******	SER, SBR, CBR, LDI, ANDI, ORI, CPI, SBCI, SUBI					**********
;*********************************************************************************************************
.undef xl			;} verhindert
.undef xh			;}  
.undef yl			;} Warnmeldung
.undef yh			;} 
.undef zl			;} des Assemblers
.undef zh			;} 
;			= r0	;    }
;			= r1	;    } Ergebnis- und Hilfsregister
;			= r2	;    } Multiplikation und Division
;			= r3	;    }
.def	s		= r4	;zur Sicherung Statusregister whrend Timer 0 ISR
.def	null		= r5	;die Null braucht man fter mal
.def	ticker		= r6	;Zhler aufwrtslaufend zur Bildung der Zeitbasis "Frames"
.def	nutzerhelligkeit= r7	;bergabewert des Nutzers
.def	sollhelligkeit	= r8	;bergabewert der Anwendung an das Fading
.def	alarmhelligkeit	= r9	;Alarmhelligkeit (liegt z.Zt. bei 80% der Sollhelligkeit)
.def	pwm		= r10	;Ist-PWM
.def	u_akku_stabil	= r11	;Zhler aufwrtslaufend zur Feststellung: Betriebsspannung war lange genug stabil
;.def			= r12	;frei
;.def			= r13	;frei
;.def			= r14	;frei
;.def			= r15	;frei

.def	var10		= r16	;}
.def	var11		= r17	;} Notitzblock 1
.def	var12		= r18	;}
.def	var20		= r19	; }
.def	var21		= r20	; } Notitzblock 2
.def	var22		= r21	; }
.def	frame		= r22	;   } 				{ Bildung der Zeitbasis "Sekunde"
.def	frame_piepser	= r23	;   } Zhler aufwrtslaufend	{ Piepstonerzeugung
.def	frame_schalter	= r24	;   }			zur	{ Erkennung Schalterkommando lang oder kurz
.def	sekunde		= r25	;   }				{ Bildung der Zeitbasis "SOS-Intervall"
.def	fader		= r26	;Nachfhrungsrate Ist- an Sollhelligkeit:  Dimmer in Stufen/Frame
				;1 = langsam bis 255 = schnell		   Softstart in Stufen/5ms
				;0 = Fading steht			   Helligkeitsnderung in Stufen/5ms
.def	flags		= r27	;erstes  Flagregister
.def	flags1		= r28	;zweites Flagregister fr Menbedienung und Benutzeroptionen
.def	piepsermaske	= r29	;Steuerregister fr den Piepser
;.def			= r30	;frei
.def	lc		= r31	;Schleifenzhler fr Software-Multiplikation/Division

;*********************************************************************************************************
;Register Flags
;BIT      7           6           5           4           3           2           1           0
;       Alarme      neue        Dimm        Dimmer       SOS       Vorwarn      Wasser      Power
;      gesperrt    Messung    Richtung     gestoppt     Modus      spannung    einbruch    schalter
;                angefordert   heller                              erkannt     erkannt       ein
;*********************************************************************************************************
;Bitdefinitionen

.equ	alarmsperre	= 7	;1 = Alarmierungen gesperrt
.equ	neue_messung	= 6	;1 = neue Messung Betriebspannung angefordert
.equ	heller_dimmen	= 5	;0 = dunkler dimmen	1 = heller dimmen
.equ	dimmer_steht	= 4	;0 = Dimmer luft	1 = Dimmer ist gestoppt
.equ	sos		= 3	;			1 = SOS-Modus aktiv
.equ	warnspannung	= 2	;			1 = Entladeende Akku erwartet
.equ	wassereinbruch	= 1	;			1 = Wassereinbruch erkannt
.equ	schalter_ein	= 0	;0 = Powerschalter aus	1 = Powerschalter ein

;*********************************************************************************************************
;Register Flags1
;BIT      7           6           5           4           3           2           1           0
;    Anforderung     frei       Wasser      Brenner     Dimm    /             Mennummer           /
;     Abschalt                 einbruch    Leistung     Modus   /            binr gezhlt         /
;  Spannung lernen             inaktiv     konstant     aktiv   /                                  /
;*********************************************************************************************************
;Bitdefinitionen

.equ	lerne_abschalt	= 7	;Anforderung lerne Abschalt-Spannung
;.equ			= 6	;frei fr eine mgliche Option 4
.equ	kein_wasser	= 5	;Wassereinbruchprfung inaktiv (z.B. fr berwasserlampen)
.equ	leistung_konst	= 4	;auf konstante Brennerleistung regeln
.equ	modus_dimmen	= 3	;stufenlose Helligkeitseinstellung
.equ	menuebit_2	= 2	; }
.equ	menuebit_1	= 1	; } 0 = Normalbetrieb
.equ	menuebit_0	= 0	; } Menbits mssen rechtsbndig definiert bleiben

.equ	alle_optionsbits= (1<<kein_wasser)|(1<<leistung_konst)|(1<<modus_dimmen)
.equ	alle_menuebits	= (1<<menuebit_2)|(1<<menuebit_1)|(1<<menuebit_0)

;Menpunkt	0 = Normalbetrieb der Lampe, alle anderen Punkte sind Lernmodus
;		1 = Betriebsart: Helligkeit in Stufen schalten oder stufenlos dimmen
;		2 = Leistungsnachregelung bei schwankender Akkuspannung: aktiv oder inaktiv
;		3 = Wassereinbruchprfung aktiv oder inaktiv
;		4 = neue Abschaltspannung lernen
;		5 = Fabrikeinstellung Abschaltspannung laden

;*********************************************************************************************************
;Register Piepser Men
;BIT      7           6           5           4           3           2           1           0
;             Anzahl        /               Anzahl              /               Anzahl
;         lange Piepser     /           kurze Piepser           /           kurze Piepser
;     Optionsignalisierung  /       Optionsignalisierung        /   Men- und Alarmsignalisierung
;*********************************************************************************************************
;Bitdefinitionen
;			= 7	;    Bit 1 } Anzahl lange Options- oder Alarmpiepser
.equ	lange_piepser	= 6	;    Bit 0 }
;			= 5	;  Bit 2 }
;			= 4	;  Bit 1 } Anzahl kurze Optionspiepser
.equ	kurze_piepser	= 3	;  Bit 0 }
;			= 2	;Bit 2 }
;			= 1	;Bit 1 } Anzahl kurze Men- oder Alarmpiepser
.equ	menue_piepser	= 0	;Bit 0 }

.equ	alle_kurzen_piepser = 0b00111000
.equ	alle_menue_piepser  = 0b00000111

;*********************************************************************************************************
;*******	Belegung des SRAM Speicherbereiches (128 Byte vorhanden)			**********
;*********************************************************************************************************
.dseg				;12 + Stackpointer verwendet
akkuspannung:	.byte	2	;letzter Messwert Akkuspannung
filter_akku:	.byte	3	;Speicher Mittelwertbildung Akkuspannung, ein Nachkommabyte
filter_akku_alt:.byte	3	;Mittelwert Akkuspannung, eine Sekunde alt
filter_wasser:	.byte	3	;Speicher Mittelwertbildung Wassereinbruchsensor, ein Nachkommabyte
u_vorwarnung:	.byte	2	;Vorwarnspannung
u_abschaltung:	.byte	2	;Abschaltspannung
scratch:	.byte	2	;Zwischenablage

;*********************************************************************************************************
;*******	Programmstart									**********
;*********************************************************************************************************
.cseg
.org	0x0000
	rjmp	reset			;Resetvektor
	reti				;externer Int0
	reti				;Potentialwechsel
	reti				;Timer1 compare match
	rjmp	timer_1_ovf		;Timer1 berlauf
	reti				;Timer0 berlauf
	reti				;EEPROM ready
	reti				;Analog Comparator fertig
	reti				;AD Conversion fertig
	reti				;Timer 1 CompB
	reti				;Timer 0 CompA
	reti				;Timer 0 CompB
	reti				;WDT
	reti				;USI Start
	reti				;USI OVF
reset:
;*********************************************************************************************************
;*******	Initialisierung									**********
;*********************************************************************************************************
	ldi	var10,CalibrateOsz	;
	out	osccal,var10		;trimme auf 6,5536 MHz
	ldi	var10,ramend		;Stackpointer initialisieren
	out	spl,var10		;ist Pflicht bei Anwendung von ISRs
	clr	null			;eine "Null" vorhalten

;*********************************************************************************************************
;*******	EEPROM initialisieren								**********
;*********************************************************************************************************
	rcall	teste_eeprom		;EEPROM auf Korruption durchsuchen und reparieren
	ldi	var11,adr_eeprom	;wenn in der erste Zelle
	rcall	eeread			;
	ldi	var11,0xaa		;"aa" steht
	cpse	var10,var11		;ist das EEPROM schon initialisiert
	rcall	init_eeprom		;sonst EEPROM initialisieren

;*********************************************************************************************************
;*******	Initialisierung AD-Wandler							**********
;*******	ADC3/mux[1:0]/PB3=Wasser  ADC2/mux[1]/PB4=Akku					**********
;*********************************************************************************************************
	ldi	var10,(1<<REFS2)|(1<<REFS1)|(1<<MUX1);interne Referenz 2,56 Volt ohne Puffer-Kondensator
	out	admux, var10		;Ergebnis rechtsbndig, PortB4 als Eingang
	ldi	var10,(1<<ADEN)|(1<<ADPS2)|(1<<ADPS1)
	out	adcsra,var10		;Einzelmessung, ADC Interrupt aus, CK/64 (102,4 kHz)
	ldi	var10,(1<<adc1d)|(1<<adc0d)
	out	didr0,var10		;unbenutzte AD-Eingnge abschalten

;*********************************************************************************************************
;*******	Initialisierung Port								**********
;*********************************************************************************************************
	ldi	var10,(1<<a_piepser)|(1<<E_lernen)
	out	portb,var10		;Buzzer ausschalten, PullUp "Lernen-Eingang" zuschalten
	ldi	var10,(1<<a_piepser)|(1<<a_pwm)
	out	ddrb,var10		;Ausgnge Buzzer und PWM setzen

;*********************************************************************************************************
;*******	Initialisierung Timer 1: Zeitbasis, Rampe Software-PWM				**********
;*********************************************************************************************************
	ldi	var10,(1<<CS13)|(1<<CS10);CK/(256*256) = 100 Hz berlauf
	out	tccr1,var10		;
	ldi	var10,(1<<toie1)	;Overflow-IRQ erlauben
	out	timsk,var10		;

;*********************************************************************************************************
;*******	Initialisierung	Betriebsart, Grenzwerte und Filter				**********
;*********************************************************************************************************
	rcall	lade_eeprom		;lade Optionen und Abschaltspannung aus dem EEPROM
	rcall	rechne_vorwarnspannung	;Vorwarnspannung rechnen und setzen  (= Abschaltspannung * 1,07)
	rcall	init_filter_akku	;Filter Akkuspannung und
	rcall	init_filter_wasser	;Wassereinbruch mit Defaultwerten laden
;*********************************************************************************************************
;*******	Register vorbereiten								**********
;*********************************************************************************************************
	ldi	flags,(1<<alarmsperre)|(1<<dimmer_steht)
	clr	ticker			; }
	clr	frame			; } Zeitregister
	clr	sekunde			; } lschen
	clr	frame_schalter		; }
	clr	piepsermaske		;keinen Pieps bitte
	clr	sollhelligkeit		;
	clr	pwm			;
	ldi	var10,100*255/k_0	;100% Starthelligkeit
	mov	nutzerhelligkeit,var10	;
	ldi	fader,2560/fading_start	;Softstart

;*********************************************************************************************************
;*******	Behandlung Programmiermodus							**********
;*********************************************************************************************************
	sbis	pinb,e_lernen		;wenn der Programmierpin nicht auf Masse liegt
	rjmp	ende_prog_einstieg	;
	andi	flags1,~alle_menuebits	;
	ori	flags1,(1<<menuebit_0)	;in den Programmiermodus Menpunkt 1 setzen
	rcall	piepscode_anfordern	;
ende_prog_einstieg:
	sei				;Interrupts freigeben

;*********************************************************************************************************
;*******	Hauptschleife									**********
;*********************************************************************************************************
start:
;*********************************************************************************************************
;*******	Software-PWM									**********
;*********************************************************************************************************
	in	var10,tcnt1		;
	cp	var10,pwm		;wenn der Timmerwert (Rampe) die Vorgabe-Helligkeit erreicht
	brlo	ende_pwm		;
	cbi	portb,a_pwm		;Brenner abschalten
ende_pwm:
;*********************************************************************************************************
	sbrs	flags,neue_messung	;nach jedem Timer 0 berlauf
	rjmp	start			;
;*********************************************************************************************************
;*******	Durchlauf hier 100 mal pro Sekunde						**********
;*******	Messung der Betriebsspannung und Schalterabfrage				**********
;*********************************************************************************************************
	cbr	flags,(1<<schalter_ein)	;
	rcall	messe_spannung		;hole Momentanwert der Betriebsspannung
	cpi	var10,byte1(schwelle_schalter_ein*1024/2560)
	ldi	var12,byte2(schwelle_schalter_ein*1024/2560)
	cpc	var11,var12		;
	brlo	ende_schalterabfrage	;Hauptschalter ist geffnet
	ori	flags,(1<<schalter_ein)	;Hauptschalter ist geschlossen
	sts	akkuspannung+0,var10	;Messwert Betriebsspannung weglegen
	sts	akkuspannung+1,var11	;
	rcall	bediene_filter_spannung	;Betriebsspannung in den Filter bergeben
ende_schalterabfrage:
	andi	flags,~(1<<neue_messung);erledigt
	mov	var10,flags1		;
	andi	var10,alle_menuebits	;
	brne	pc+2			;im Normalmodus
	sbi	portb,a_pwm		;Brenner jetzt einschalten
;*********************************************************************************************************
;*******	Fading										**********
;*******	die PWM wird ber eine Rampe der SOLL-Helligkeit nachgefhrt			********** 
;*******	"Fader" bestimmt die Geschwindigkeit des Helligkeitsbergangs			**********
;*******	der SOS-Modus bergibt direkt an die PWM					**********
;*********************************************************************************************************
	mov	var10,fader		;sichern
beginn_fading:
	cp	pwm,sollhelligkeit	;Ist-Helligkeit hat Soll-Helligkeit erreicht?
	breq	ende_neue_messung	;ja, nichts weiter zu tun
	tst	var10			;alle Durchlufe fr das Fading schon abgearbeitet ?
	breq	ende_neue_messung	;ja, raus hier
	dec	var10			;sonst ein Durchlauf weniger
	cp	pwm,sollhelligkeit	;ist die Ist-PWM zu niedrig ?
	brlo	pwm_zu_klein		;dann dort weiter
;pwm_zu_gro:
	dec	pwm			;verringere um eins
	rjmp	beginn_fading		;bis das Fadingintervall stimmt
pwm_zu_klein:
	inc	pwm			;erhhe um eins
	rjmp	beginn_fading		;bis das Fadingtempo stimmt
ende_neue_messung:
;*********************************************************************************************************
;*******	Zeitzhlung									**********
;*******	Ticker wird von Timer1-ISR mit 100 Hz hochgezhlt				**********
;*********************************************************************************************************
	mov	var10,ticker		;sichern
	cpi	var10,4			;prfe ob Frame um
	brlo	start			;wenn nicht, Zeitzhlung verlassen
	clr	ticker			;sonst Ticker rcksetzen und weiter
;*********************************************************************************************************
;*******	Durchlauf hier jedes Frame							**********
;*********************************************************************************************************
;*********************************************************************************************************
;*******	Helligkeitssteuerung SOS							**********
;*********************************************************************************************************
	sbrs	flags,sos		;wenn SOS-Modus aktiv ist
	rjmp	kein_sos_aktiv		;

	ldi	var10,(max_helligkeit*255/k_0);volle Helligkeit
	mov	pwm,var10		;
	mov	sollhelligkeit,var10	;

	cpi	sekunde,3		;
	brlo	kleiner_drei_sek	;
	cpi	sekunde,10		;
	brlo	kleiner_zehn_sek	;
	cpi	sekunde,13		;
	brlo	kleiner_dreizehn_sek	;
licht_aus:
	ldi	var10,1			;Licht dunkel (nur vorheizen)
	mov	pwm,var10		;
	mov	sollhelligkeit,var10	;
	rjmp	ende_schalter		;
kleiner_drei_sek:
	cpi	frame,13		;zwischen Sekunde 0 und 2
	brlo	licht_aus		;wird in der ersten Sekundenhlfte
	rjmp	ende_schalter		;das Licht dunkelgetastet
kleiner_zehn_sek:
	sbrc	sekunde,0		;zwischen Sekunde 3 und 9 wird in den
	rjmp	licht_aus		;ungeraden Sekunden das Licht dunkelgetastet
	rjmp	ende_schalter		;
kleiner_dreizehn_sek:
	cpi	frame,12		;zwischen Sekunde 10 und 12
	brsh	licht_aus		;wird in der zweiten Sekundenhlfte
	rjmp	ende_schalter		;das Licht dunkelgetastet
kein_sos_aktiv:
;*********************************************************************************************************
;*******	Hauskeeping									**********
;*********************************************************************************************************
	mov	sollhelligkeit,nutzerhelligkeit		;
	sbrc	flags1,leistung_konst			;mit Option Regelung auf konstante Leistung
	rcall	leisungsnachfuehrung			;bitteschn
	mov	var10,sollhelligkeit			;
	ldi	var11,(alm_Helligkeit*255/100)		;80% der Sollhelligkeit wird die Alarmhelligkeit
	rcall	mul8u					;
	mov	alarmhelligkeit,r1			;
;*********************************************************************************************************
;*******	Dimmersteuerung									**********
;*********************************************************************************************************
	sbrs	flags1,modus_dimmen			;im Schaltmodus
	rjmp	ende_dimmersteuerung			;oder
	sbrc	flags,dimmer_steht			;bei abgelaufenem Dimmvorgang
	rjmp	ende_dimmersteuerung			;oder
	tst	fader					;bei abgelaufenem Fader
	breq	ende_dimmersteuerung			;nichts anfassen

	mov	var10,nutzerhelligkeit			;sichere Register
	mov	var11,fader				;
dimmer_1:
	sbrs	flags,heller_dimmen			;Dimmrichtung "heller"
	rjmp	dimme_heller				;hier lang
;dimme_dunkler:
	cpi	var10,((min_helligkeit+1)*255/k_0)	;
	brlo	stop_dimmer				;
	dec	nutzerhelligkeit			;
	rjmp	teste_fader				;
dimme_heller:
	cpi	var10,(max_helligkeit*255/k_0)		;
	brsh	stop_dimmer				;
	inc	nutzerhelligkeit			;
teste_fader:
	dec	var11					;
	breq	ende_dimmersteuerung			;
	rjmp	dimmer_1				;
stop_dimmer:
	sbr	flags,(1<<dimmer_steht)			;Dimmer anhalten
ende_dimmersteuerung:
;*********************************************************************************************************
;*******	Signalgeber									**********
;*********************************************************************************************************
	tst	piepsermaske				;wenn Piepser angefordert sind
	breq	ende_piepser				;

	mov	var10,piepsermaske			;
	andi	var10,alle_menue_piepser		;
	brne	auspiepsen_menue			;erst die kurzen Menpiepser abarbeiten

	mov	var10,piepsermaske			;
	andi	var10,alle_kurzen_piepser		;
	brne	auspiepsen_kurz				;dann die kurzen Optionspiepser und

;auspiepsen_lang:
	cpi	frame_piepser,dauer_langer_pieps	;
	brlo	piepser_ein				;
	breq	piepser_aus_lang			;
	rjmp	piepser_aus				;
piepser_aus_lang:
	sbi	portb,a_piepser				;Piepser aus
	subi	piepsermaske,(1<<lange_piepser)		;ein Pieps weniger
	subi	var10,(1<<lange_piepser)		;
	breq	piepser_fertig				;alle langen Piepser erledigt
	ldi	frame_piepser,(256-pause_langer_pieps)	;
	rjmp	ende_piepser				;

auspiepsen_kurz:
	cpi	frame_piepser,dauer_kurzer_pieps	;
	brlo	piepser_ein				;
	breq	piepser_aus_kurz			;
	rjmp	piepser_aus				;
piepser_aus_kurz:
	sbi	portb,a_piepser				;Piepser aus
	subi	piepsermaske,(1<<kurze_piepser)		;ein Pieps weniger
	subi	var10,(1<<kurze_piepser)		;
	breq	piepser_fertig				;alle Men-Piepser erledigt
	ldi	frame_piepser,(256-pause_kurzer_pieps)	;
	rjmp	ende_piepser				;

auspiepsen_menue:
	cpi	frame_piepser,dauer_kurzer_pieps	;
	brlo	piepser_ein				;
	breq	piepser_aus_menue			;
	rjmp	piepser_aus				;
piepser_aus_menue:
	sbi	portb,a_piepser				;Piepser aus
	subi	piepsermaske,(1<<menue_piepser)		;ein Pieps weniger
	subi	var10,(1<<menue_piepser)		;
	breq	piepser_fertig				;alle Men-Piepser erledigt
	ldi	frame_piepser,(256-pause_kurzer_pieps)	;
	rjmp	ende_piepser				;
piepser_fertig:
	ldi	frame_piepser,(256-25)			;1 Sekunde warten bis nchste Tonfolge
	rjmp	ende_piepser				;
piepser_ein:
	cbi	portb,a_piepser				;Piepser ein
	rjmp	ende_piepser				;
piepser_aus:
	;sbi	portb,a_piepser				;Piepser aus
ende_piepser:
;*********************************************************************************************************
;*******	Helligkeit bei Unterspannung oder Wassereinbruch				**********
;*******	zustzliche akustische Signalisierung nur bei Wassereinbruch			**********
;*********************************************************************************************************
	mov	var10,flags				;sichern
	andi	var10,(1<<warnspannung)|(1<<wassereinbruch)
	breq	ende_alarm				;ohne einen Alarm raus hier

	mov	var10,sekunde				;sonst
	andi	var10,0b00000011			;alle 4 Sekunden 
	brne	ende_alarm				;

	cpi	frame,10				;kurz (10 * 40ms = 400ms)
	brsh	ende_alarm				;
	mov	sollhelligkeit,alarmhelligkeit		;das Licht abdunkeln

	sbrs	flags,wassereinbruch			;nur bei Wassereinbruch
	rjmp	ende_alarm				;

	cpi	frame,0					;
	brne	ende_alarm				;
	ldi	piepsermaske,(2<<lange_piepser)		;2 lange Piepser anfordern
	clr	frame_piepser				;
ende_alarm:
;*********************************************************************************************************
;*******	Schalterbehandlung								**********
;*******	Beim ffnen des Schalters wird der Zhler "frame_schalter" gestartet.		**********
;*******	Wenn er einmal losgelaufen ist wird er auch bei geschlossenem Schalter		**********
;*******	weiterbedient, um Kontaktprellen zu erkennen.				.	**********
;*******											**********
;*******	Beim Schlieen des Schalters wird je nach Dauer seit letztem ffnen verfahren.	**********
;*******											**********
;*******				{ < Prellen = keine Aktion				**********
;*******	Dauer			{ < Dauer Kurzkommando = Kurzkommando ausfhren		**********
;*******	Schalterkommando	{ < Dauer Langkommando = Langkommando ausfhren		**********
;*******				{ > Dauer Langkommando = Lampe abschalten		**********
;*******											**********
;*******	Nach einem ausgefhrten Kommando (lang oder kurz) wird der Zhler gestoppt	**********
;*********************************************************************************************************
	sbrc	flags,schalter_ein	;
	rjmp	teste_laenge		;Schalter ist geschlossen
;*********************************************************************************************************
;*******	Schalter ist geffnet								**********
;*********************************************************************************************************
	inc	frame_schalter		;Zhler zum Messen der Dauer des Schalterkommandos bedienen
	cpi	frame_schalter,dauer_langkommando/40;Ende Zeitfenster fr Langkommando
	brlo	ende_schalter		;
abschalten:
	cli				;erreicht, kein Wecken mehr durch Interrupt
	sbi	portb,a_piepser		;Piepser abschalten
	cbi	portb,a_pwm		;Brenner abschalten
	rjmp	abschalten		;wirklich
;*********************************************************************************************************
;*******	Schalter ist geschlossen oder wieder geschlossen				**********
;*********************************************************************************************************
teste_laenge:
	tst	frame_schalter		;den einmal gestarteten Zhler auch
	breq	pc+2			;
	inc	frame_schalter		;beim Prellen des Kontaktes weiter bedienen
	cpi	frame_schalter,prellen	;mgliche Prellzeit um?
	brlo	ende_schalter		;
	cpi	frame_schalter,dauer_kurzkommando/40;Ende Zeitfenster fr Kurzkommando
	brlo	kurze_dauer		;
;lange_dauer:
	rcall	kommando_lang		;Schalterkommando "lang" ausfhren
	clr	frame_schalter		;Zhler stoppen
	rjmp	ende_schalter		;
kurze_dauer:
	rcall	kommando_kurz		;Schalterkommando "kurz" ausfhren
	clr	frame_schalter		;Zhler stoppen
	rjmp	ende_schalter		;
ende_schalter:
;*********************************************************************************************************
;*******	Fortsetzung Zeitzhlung								**********
;*********************************************************************************************************
	inc	frame_piepser		;Zhler Piepsersteuerung bedienen
	inc	frame			;erhhe Frame
	cpi	frame,25		;prfe ob Sekunde um
	brsh	neue_sekunde		;
	rjmp	start			;wenn nicht, Zeitzhlung verlassen
neue_sekunde:
	clr	frame			;sonst Frames rcksetzen und weiter
;*********************************************************************************************************
;*******	Durchlauf hier jede Sekunde							**********
;*********************************************************************************************************
;*********************************************************************************************************
;*******	Spannungsmessung und Alarmprfung						**********
;*********************************************************************************************************
	mov	var10,flags1		;
	andi	var10,alle_menuebits	;im Programmiermodus
	brne	keine_alarmpruefung	;keine Alarme
	sbrc	flags,alarmsperre	;und bei Alarmsperre
	rjmp	keine_alarmpruefung	;
	sbrc	flags,sos		;bei SOS-Ausgabe
	rjmp	keine_alarmpruefung	;keine Alarme
	rcall	teste_auf_unterspannung	;
	sbrs	flags1,kein_wasser	;wenn Option freigegeben
	rcall	teste_wassereinbruch	;auf Wassereinbruch prfen
keine_alarmpruefung:
;*********************************************************************************************************
;*******  Abschaltspannung lernen								**********
;*********************************************************************************************************
	mov	var10,flags1		;
	andi	var10,alle_menuebits+(1<<lerne_abschalt)
	cpi	var10,4+(1<<lerne_abschalt);im Men 4 bei Anforderung "Abschaltspannung lernen"
	brne	ende_abschaltspannung_lernen

	lds	var20,filter_akku_alt+0	;hole letzte gefilterte Betriebsspannung
	lds	var21,filter_akku_alt+1	;
	lds	var22,filter_akku_alt+2	;

	lds	var10,filter_akku+0	;hole gefilterte Betriebsspannung
	lds	var11,filter_akku+1	;
	lds	var12,filter_akku+2	;
;*******  Betragsbildung
	sub	var20,var10		;bilde die Differenz 
	sbc	var21,var11		; 
	sbc	var22,var12		;

	brcc	u_ist_positiv		;bei negativem Ergebnis

	com	var22			;erhlt man den Absolutbetrag indem
	com	var21			;das Zweierkomplement gebildet und
	neg	var20			;1 dazu addiert wird
	sbci	var21,byte2(-1)		;
	sbci	var22,byte3(-1)		;
u_ist_positiv:
;*******  Ende Betragsbildung
.equ	delta_a	= 20				;20 mV
	cpi	var20,byte1(delta_a*k_1/10)	;ist der alte Filterwert nah am aktuellen?
	ldi	var10,byte2(delta_a*k_1/10)	;
	cpc	var21,var10		;
	ldi	var10,byte3(delta_a*k_1/10)	;
	cpc	var22,var10		;
	brsh	filter_akku_nicht_stabil;nein

	inc	u_akku_stabil		;sonst wenn die
	ldi	var10,5			;Spannung  Sekunden lang 5stabil ist
	cp	u_akku_stabil,var10	;
	brlo	ende_abschaltspannung_lernen
					;speichere var12:var11, "filter_akku" nach EEPROM
	mov	var10,var11		;Lowbyte umkopieren
	ldi	var11,adr_eeprom+1	;lade EEPROM-Adresse fr LowByte Abschaltspannung
	rcall	eewrite_3		;und sichern
	mov	var10,var12		;Highbyte Messwert vorlegen
	ldi	var11,adr_eeprom+2	;lade EEPROM-Adresse fr HighByte Abschaltspannung
	rcall	eewrite_3		;und sichere

	andi	flags1,~(1<<lerne_abschalt)
	ldi	piepsermaske,(4<<menue_piepser)+(1<<lange_piepser);4 kurze und einen langen Pieps anfordern
	clr	frame_piepser		;
	rjmp	ende_abschaltspannung_lernen
filter_akku_nicht_stabil:
	clr	u_akku_stabil		;
	ldi	piepsermaske,(1<<kurze_piepser);einen kurzen Pieps anfordern
	clr	frame_piepser		;
ende_abschaltspannung_lernen:
	lds	var10,filter_akku+0	;aktueller Wert wird alter Wert
	lds	var11,filter_akku+1	;
	lds	var12,filter_akku+2	;

	sts	filter_akku_alt+0,var10	;
	sts	filter_akku_alt+1,var11	;
	sts	filter_akku_alt+2,var12	;
;*********************************************************************************************************
;*******	Fortsetzung Zeitzhlung								**********
;*********************************************************************************************************
	inc	sekunde			;erhhe Sekunde
	cpi	sekunde,16		;prfe ob SOS-Zykluszeit um
	brsh	neuer_zeitrahmen	;
	rjmp	start			;wenn nicht, Zeitzhlung verlassen
neuer_zeitrahmen:
	clr	sekunde			;sonst Sekunde rcksetzen und weiter
;*********************************************************************************************************
;*******	Durchlauf hier alle 16 Sekunden							**********
;*********************************************************************************************************
	andi	flags,~(1<<alarmsperre)
	rjmp	start

;*********************************************************************************************************
;*******	Timer1 berlauf mit 100 Hz							**********
;*******	Erzeugung Zeitbasis						ng		**********
;*********************************************************************************************************
Timer_1_ovf:
	in	s,sreg			;Statusregister sichern
	inc	ticker			;Ticker bedienen
	ori	flags,(1<<neue_messung)	;Betriebsspannungsmessung anfordern
	cbi	portb,a_pwm		;Brenner abschalten
	out	sreg,s			;Statusregister wiederherstellen
	reti

.include "unterprogramme.inc"		;Unterprogramme
.include "arithmetikroutinen.inc"	;Rechenoperationen
